package com.ibm.oti.shared;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 1998, 2010  All Rights Reserved
 */

import java.net.URL;

/**
 * Shared Class Helper API that stores and finds classes using URL paths.<p>
 * <p>
 * - Description<p>
 * A SharedClassURLHelper is obtained by calling getSharedClassURLHelper(ClassLoader) on a SharedClassHelperFactory.<br>
 * The SharedClassURLHelper is designed for ClassLoaders which load classes from many different locations, without the concept of a classpath.
 * A ClassLoader may find and store classes in the cache using any URL.<p>
 * <p>
 * - Usage<p>
 * The ClassLoader should call findSharedClass after looking in its local cache and asking its parent (if one exists).
 * If findSharedClass does not return null, the ClassLoader should call defineClass on the byte[] returned.<br>
 * The ClassLoader should call storeSharedClass immediately after a class has been defined, but NOT if the class being defined was loaded from the shared cache.<br>
 * The ClassLoader is reponsible for coordinating the creation and use of partition Strings, if partitions are required (see below).<br>
 * Classes can only be stored using URLs which have "file" or "jar" protocols and which refer to existing resources.
 * <p>
 * - Dynamic Cache Updates<p>
 * Since the shared cache is persistent beyond the lifetime of a JVM, classes in the shared cache may become out-of-date ("stale").<br>
 * Classes in the cache are automatically kept up-to-date by default:<br>
 * If findSharedClass is called for a class which exists in the cache but which has been updated on the filesystem since it was stored,
 * then the old version in the cache is automatically marked stale and findSharedClass will return null.
 * The ClassLoader should then store the updated version of the class.<br>
 * Note that jar/zip updates will cause all cache entries loaded from that jar/zip to be marked stale.<br>
 * (This behaviour can be disabled using the correct command-line option. See -Xshareclasses:help)<br>
 * It is also assumed that the ClassLoader will maintain a read lock on jar/zip files opened during its lifetime, preventing their modification.
 * This prevents the cache from having to constantly check for updates.
 * <p>
 * - Partitions<p>
 * A partition may be used when finding or storing a class, which allows modified versions of the same class
 *   to be stored in the cache, effectively creating "partitions" in the cache.<br>
 * Partitions are designed for bytecode modification such as the use of Aspects. It is the responsibility of the ClassLoader
 *   to create partitions which describe the type of modification performed on the class bytes.<br>
 * If a class is updated on the filesystem and automatic dynamic updates are enabled, then all versions of the class across
 *   all partitions will be marked stale.
 * <p>
 * - Class Metadata<p>
 * A ClassLoader may create metadata when loading and defining classes, such as a jar manifest or security data.
 * None of this metadata may be stored in the cache, so if a ClassLoader is finding classes in the shared cache, it must load
 *   any metadata that it needs from disk before defining the classes.
 * <p>
 * - Security<p>
 * A SharedClassHelper will only allow classes to be stored in the cache which were defined by the ClassLoader that owns the SharedClassHelper.<br>
 * If a SecurityManager is installed, SharedClassPermissions must be used to permit read/write access to the shared class cache.
 * Permissions are granted by ClassLoader classname in the java.policy file and are fixed when the SharedClassHelper is created.<br>
 * Note also that if the createClassLoader RuntimePermission is not granted, ClassLoaders cannot be created
 * which in turn means that SharedClassHelpers cannot be created.
 * <p>
 * - Compatibility with other SharedClassHelpers<p>
 * Classes stored using the SharedClassURLHelper can be retrieved using the SharedClassURLClasspathHelper and vice-versa.
 * This is also true for partitions which can be used across these two helpers.
 * <p>
 * @author		OTI
 * @version		initial
 *
 * @see SharedClassURLHelper
 * @see SharedClassHelperFactory
 * @see SharedClassPermission
 */
public interface SharedClassURLHelper extends SharedClassHelper {

	/**
	 * Find a class in the shared cache using a specific URL and class name.<p>
	 * A class will be returned if it was stored against the same URL.<br>
	 * Null is returned if the class cannot be found or if it is stale (see "Dynamic Cache Updates" above).<br>
	 * To obtain an instance of the class, the byte[] returned must be passed to defineClass by the caller ClassLoader.<br>
	 * Also returns false if the URL at foundAtIndex is not a file URL or if the resource it refers to does not exist.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @param 		path URL
	 * 					A URL path. Cannot be null.
	 *
	 * @param 		className String
	 * 					The name of the class to be found
	 *
	 * @return		byte[]
	 * 					A byte array describing the class found, or null.
	 */
	public byte[] findSharedClass(URL path, String className);

	/**
	 * Find a class in the shared cache using a specific URL, class name and user-defined partition (see "Partitions" above).<p>
	 * A class will only be returned if it was stored against the same URL and using the same partition.
	 * Null is returned if the class cannot be found or if it is stale (see "Dynamic Cache Updates" above).<br>
	 * To obtain an instance of the class, the byte[] returned must be passed to defineClass by the caller ClassLoader.<br>
	 * Also returns false if the URL at foundAtIndex is not a file URL or if the resource it refers to does not exist.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @param 		partition String.
	 * 					User-defined partition if finding modified bytecode (see "Partitions" above).
	 * 					Passing null is equivalent of calling non-partition findSharedClass call.
	 *
	 * @param 		path URL.
	 * 					A URL path. Cannot be null.
	 *
	 * @param 		className String.
	 * 					The name of the class to be found
	 *
	 * @return		byte[].
	 * 					A byte array describing the found class, or null.
	 */
	public byte[] findSharedClass(String partition, URL path, String className);

	/**
	 * Store a class in the shared cache using the URL location it was loaded from.<p>
	 * The class being stored must have been defined by the caller ClassLoader and must exist in the URL location specified.
	 * Returns true if the class is stored successfully or false otherwise.<br>
	 * Will return false if the class being stored was not defined by the caller ClassLoader.<br>
	 * Also returns false if the URL at foundAtIndex is not a file URL or if the resource it refers to does not exist.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @param 		path URL.
	 * 					The URL path that the class was loaded from. Cannot be null.
	 *
	 * @param 		clazz Class.
	 * 					The class to store in the shared cache
	 *
	 * @return		boolean.
	 * 					True if the class was stored successfully, false otherwise.
	 */
	public boolean storeSharedClass(URL path, Class clazz);

	/**
	 * Store a class in the shared cache using the URL location it was loaded from and a user-defined partition (see "Partitions" above).<p>
	 * The class being stored must have been defined by the caller ClassLoader and must exist in the URL location specified.
	 * Returns true if the class is stored successfully or false otherwise.<br>
	 * Will return false if the class being stored was not defined by the caller ClassLoader.<br>
	 * Also returns false if the URL at foundAtIndex is not a file URL or if the resource it refers to does not exist.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @param 		partition String.
	 * 					User-defined partition if storing modified bytecode (see "Partitions" above).
	 * 					Passing null is equivalent of calling non-partition storeSharedClass call.
	 *
	 * @param 		path URL.
	 * 					The URL path that the class was loaded from. Cannot be null.
	 *
	 * @param 		clazz Class.
	 * 					The class to store in the shared cache
	 *
	 * @return		boolean.
	 * 					True if the class was stored successfully, false otherwise.
	 */
	public boolean storeSharedClass(String partition, URL path, Class clazz);

	/**
	 * Minimize update checking on jar files for optimal performance<p>
	 * By default, when a class is loaded from the shared class cache, the timestamp of the container it was originally loaded from is
	 * compared with the timestamp of the actual container on the filesystem. If the two do not match, the original class is marked stale
	 * and is not returned by findSharedClass(). These checks are not performed if the container is held open by the ClassLoader.<br>
	 * If the ClassLoader does not want to open the container, but doesn't want the timestamp to be constantly checked when
	 * classes are loaded, it should call this function immediately after the SharedClassURLHelper object has been created.
	 * Once this function has been called, each container timestamp is checked once and then is not checked again for the lifetime
	 * of the SharedClassURLHelper.<br>
	 * This feature cannot be unset.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @return		boolean
	 * 					True if the feature has been successfully set, false otherwise.
	 */
	public boolean setMinimizeUpdateChecks();
}
